import unittest
from unittest.mock import ANY, MagicMock, patch

from lifeguard.controllers import (
    Response,
    load_custom_controllers,
    treat_response,
    configure_controller,
    render_template,
    Request,
    login_required,
)
from tests.fixtures.controllers.hello_controller import hello

mock_flask_request = MagicMock(name="flask_request")

MOCK_BASIC_AUTH = MagicMock(name="basic_auth")
MOCK_AUTHENTICATION_METHODS = {"basic_auth": MOCK_BASIC_AUTH}


class TestControllers(unittest.TestCase):
    @patch("lifeguard.controllers.LIFEGUARD_DIRECTORY", "tests/fixtures")
    @patch("lifeguard.controllers.logger")
    @patch("lifeguard.controllers.custom_controllers")
    def test_load_custom_controllers(self, mock_custom_controllers, mock_logger):
        load_custom_controllers()
        mock_logger.info.assert_any_call(
            "loading custom controller %s", "hello_controller"
        )
        mock_custom_controllers.add_url_rule.assert_called_with("/hello", "hello", ANY)
        self.assertEqual("hello", hello())

    @patch("lifeguard.controllers.FlaskResponse")
    @patch("lifeguard.controllers.render_template")
    def test_treat_response_with_template(
        self, mock_render_template, mock_flask_response
    ):
        mock_render_template.return_value = "template rendered"

        response = Response()
        response.template = "template.html"
        response.template_searchpath = "/templates"

        treat_response(response)

        mock_flask_response.assert_called_with(
            "template rendered", content_type="text/html", status=200
        )
        mock_render_template.assert_called_with("template.html", "/templates", data={})

    @patch("lifeguard.controllers.FlaskResponse")
    @patch("lifeguard.controllers.render_template")
    def test_treat_response_without_template(
        self, mock_render_template, mock_flask_response
    ):
        mock_render_template.return_value = "template rendered"

        response = Response()
        response.content = "body {}"
        response.content_type = "text/css"
        response.status = 404

        treat_response(response)

        mock_flask_response.assert_called_with(
            "body {}", content_type="text/css", status=404
        )
        mock_render_template.assert_not_called()

    def test_configure_controller_success_with_string(self):
        def test():
            return "str"

        self.assertEqual("str", configure_controller(test)())

    @patch("lifeguard.controllers.treat_response")
    def test_configure_controller_success_with_response(self, mock_treat_response):
        response = Response()

        def test():
            return response

        configure_controller(test)()
        mock_treat_response.assert_called_with(response)

    @patch("lifeguard.controllers.logger")
    @patch("lifeguard.controllers.traceback")
    def test_configure_controller_error(self, mock_traceback, mock_logger):
        mock_traceback.format_exc.return_value = "traceback"

        def test():
            raise Exception()

        configure_controller(test)()
        mock_logger.error.assert_called_with(
            "error on render dashboard index: %s",
            "",
            extra={"traceback": "traceback"},
        )

    @patch("lifeguard.controllers.jinja2")
    def test_render_template(self, mock_jinja2):

        template_loader = MagicMock(name="template_loader")
        mock_jinja2.FileSystemLoader.return_value = template_loader

        template_env = MagicMock(name="template_env")
        mock_jinja2.Environment.return_value = template_env

        template = MagicMock(name="template")
        template_env.get_template.return_value = template

        render_template("template", "searchpath")

        mock_jinja2.FileSystemLoader.assert_called_with(searchpath="searchpath")
        mock_jinja2.Environment.assert_called_with(loader=template_loader)
        template.render.assert_called()

    @patch("lifeguard.controllers.jinja2")
    def test_render_template_with_data(self, mock_jinja2):

        template_loader = MagicMock(name="template_loader")
        mock_jinja2.FileSystemLoader.return_value = template_loader

        template_env = MagicMock(name="template_env")
        mock_jinja2.Environment.return_value = template_env

        template = MagicMock(name="template")
        template_env.get_template.return_value = template

        render_template("template", "searchpath", data={"test": True})

        mock_jinja2.FileSystemLoader.assert_called_with(searchpath="searchpath")
        mock_jinja2.Environment.assert_called_with(loader=template_loader)
        template.render.assert_called_with(test=True)

    @patch("lifeguard.controllers.flask_request", mock_flask_request)
    def test_request_proxy(self):
        request = Request()
        mock_flask_request.json = "json"
        mock_flask_request.data = "data"
        mock_flask_request.form = "form"
        mock_flask_request.method = "method"
        mock_flask_request.args = "args"
        mock_flask_request.values = "values"

        self.assertEqual("json", request.json)
        self.assertEqual("data", request.data)
        self.assertEqual("form", request.form)
        self.assertEqual("method", request.method)
        self.assertEqual("args", request.args)
        self.assertEqual("values", request.values)

    @patch("lifeguard.controllers.AUTHENTICATION_METHODS", MOCK_AUTHENTICATION_METHODS)
    @patch("lifeguard.controllers.LIFEGUARD_CONTEXT")
    def test_login_required_wrap_controller(self, mock_lifeguard_context):

        mock_lifeguard_context.auth_method = "basic_auth"
        MOCK_AUTHENTICATION_METHODS["basic_auth"].return_value = False

        a_function = MagicMock(name="a_function")
        wrapped = login_required(a_function)

        self.assertFalse(wrapped())
        MOCK_AUTHENTICATION_METHODS["basic_auth"].assert_called_with((), {}, a_function)

    @patch("lifeguard.controllers.AUTHENTICATION_METHODS", MOCK_AUTHENTICATION_METHODS)
    @patch("lifeguard.controllers.LIFEGUARD_CONTEXT")
    def test_login_required_not_wrap_controller(self, mock_lifeguard_context):

        mock_lifeguard_context.auth_method = None

        a_function = MagicMock(name="a_function")
        a_function.return_value = 1
        wrapped = login_required(a_function)

        self.assertEqual(wrapped(), 1)
        MOCK_AUTHENTICATION_METHODS["basic_auth"].assert_not_called()



import unittest
from unittest.mock import MagicMock, patch

from lifeguard import NORMAL, PROBLEM
from lifeguard.notifications import NotificationBase, NotificationStatus
from lifeguard.actions.notifications import (
    notify_in_single_message,
    notify_in_thread,
)

mock_implementation = MagicMock(name="mock_implementation")
mock_logger = MagicMock(name="logger")


class MockNotificationBase(NotificationBase):
    def __init__(self):
        self.mock_implementation = None

    def send_single_message(self, content, settings):
        super().send_single_message(content, settings)
        self.mock_implementation.send_single_message(content, settings)

    def init_thread(self, content, settings):
        super().init_thread(content, settings)
        self.mock_implementation.init_thread(content, settings)

    def update_thread(self, thread_id, content, settings):
        super().update_thread(thread_id, content, settings)
        self.mock_implementation.update_thread(thread_id, content, settings)

    def close_thread(self, thread_id, content, settings):
        super().close_thread(thread_id, content, settings)
        self.mock_implementation.close_thread(thread_id, content, settings)


mock_notification_instance = MockNotificationBase()
mock_notification_instance.mock_implementation = mock_implementation


NOTIFICATION_METHODS = [mock_notification_instance]


class TestActionNotification(unittest.TestCase):
    def setUp(self):

        self.mock_validation_response = MagicMock(name="mock_validation_response")
        self.mock_validation_response.settings = None
        self.mock_settings = {}

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", [])
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_not_send_notify_in_single_message(self):

        notify_in_single_message(self.mock_validation_response, self.mock_settings)
        mock_implementation.send_single_message.assert_not_called()

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", NOTIFICATION_METHODS)
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_send_notify_in_single_message(self):
        self.mock_validation_response.settings = {"notification": {"notify": True}}
        self.mock_validation_response.details = {"status": "PROBLEM"}

        notify_in_single_message(self.mock_validation_response, self.mock_settings)

        mock_implementation.send_single_message.assert_called_with(
            '{"status": "PROBLEM"}', {}
        )

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", NOTIFICATION_METHODS)
    @patch("lifeguard.actions.notifications.json")
    @patch("lifeguard.actions.notifications.NotificationRepository")
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_send_notify_in_thread_not_problem_and_not_last_notification(
        self, mock_notification_repository, mock_json
    ):
        notification_repository_instance = MagicMock(name="repository")
        mock_notification_repository.return_value = notification_repository_instance
        notification_repository_instance.fetch_last_notification_for_a_validation.return_value = (
            None
        )

        self.mock_validation_response.status = NORMAL

        notify_in_thread(self.mock_validation_response, self.mock_settings)

        mock_json.dumps.assert_not_called()

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", NOTIFICATION_METHODS)
    @patch("lifeguard.actions.notifications.json")
    @patch("lifeguard.actions.notifications.NotificationRepository")
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_send_notify_open_thread(
        self, mock_notification_repository, mock_json
    ):
        notification_repository_instance = MagicMock(name="repository")
        mock_notification_repository.return_value = notification_repository_instance
        notification_repository_instance.fetch_last_notification_for_a_validation.return_value = (
            None
        )

        mock_json.dumps.return_value = "{}"

        self.mock_validation_response.status = PROBLEM
        self.mock_validation_response.details = {}

        notify_in_thread(self.mock_validation_response, self.mock_settings)

        mock_implementation.init_thread.assert_called_with("{}", {})

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", NOTIFICATION_METHODS)
    @patch("lifeguard.actions.notifications.json")
    @patch("lifeguard.actions.notifications.NotificationRepository")
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_send_notify_close_thread(
        self, mock_notification_repository, mock_json
    ):
        notification_repository_instance = MagicMock(name="repository")
        mock_notification_repository.return_value = notification_repository_instance
        notification_repository_instance.fetch_last_notification_for_a_validation.return_value = NotificationStatus(
            "validation", {"not implemented": "thread"}
        )

        mock_json.dumps.return_value = "{}"

        self.mock_validation_response.status = NORMAL
        self.mock_validation_response.details = {}

        notify_in_thread(self.mock_validation_response, self.mock_settings)

        mock_implementation.close_thread.assert_called_with("thread", "{}", {})

    @patch("lifeguard.actions.notifications.NOTIFICATION_METHODS", NOTIFICATION_METHODS)
    @patch("lifeguard.actions.notifications.json")
    @patch("lifeguard.actions.notifications.NotificationRepository")
    @patch("lifeguard.actions.notifications.logger", mock_logger)
    def test_should_send_notify_update_thread(
        self, mock_notification_repository, mock_json
    ):
        notification_repository_instance = MagicMock(name="repository")
        mock_notification_repository.return_value = notification_repository_instance
        notification_repository_instance.fetch_last_notification_for_a_validation.return_value = NotificationStatus(
            "validation", {"not implemented": "thread"}
        )

        mock_json.dumps.return_value = "{}"

        self.mock_validation_response.status = PROBLEM
        self.mock_validation_response.details = {}

        notify_in_thread(self.mock_validation_response, self.mock_settings)

        mock_implementation.update_thread.assert_called_with("thread", "{}", {})



import unittest
from unittest.mock import patch, MagicMock

from lifeguard.scheduler import configure_validations

mock_ref = MagicMock(name="ref")


class TestScheduler(unittest.TestCase):
    @patch(
        "lifeguard.scheduler.VALIDATIONS",
        {"example": {"ref": mock_ref, "schedule": {"every": {"minutes": 1}}}},
    )
    @patch("lifeguard.scheduler.schedule")
    def test_configure_validations_every_minute(self, mock_schedule):
        configure_validations()

        mock_schedule.every.assert_called_with(1)

    @patch(
        "lifeguard.scheduler.VALIDATIONS",
        {"example": {"ref": mock_ref, "schedule": {"every": {"hours": 1}}}},
    )
    @patch("lifeguard.scheduler.schedule")
    def test_configure_validations_every_hour(self, mock_schedule):
        configure_validations()

        mock_schedule.every.assert_called_with(1)

    @patch(
        "lifeguard.scheduler.VALIDATIONS",
        {"example": {"ref": mock_ref, "schedule": {"every": {"blah": 1}}}},
    )
    @patch("lifeguard.scheduler.schedule")
    def test_configure_validations_with_invalid_time_period(self, mock_schedule):
        configure_validations()

        mock_schedule.every.assert_not_called()



import unittest
from unittest.mock import MagicMock, patch

from lifeguard.auth import (
    basic_auth_login,
    basic_auth_login_required_implementation,
    AUTHENTICATION_METHODS,
)
from lifeguard.context import LifeguardContext

lifeguard_context = LifeguardContext()
lifeguard_context.users = [{"username": "test", "password": "pass"}]


class TestAuth(unittest.TestCase):
    @patch("lifeguard.auth.LIFEGUARD_CONTEXT", lifeguard_context)
    def test_auth_valid_user(self):
        header = "Basic dGVzdDpwYXNz"

        self.assertListEqual(
            basic_auth_login(header), [{"username": "test", "password": "pass"}]
        )

    @patch("lifeguard.auth.LIFEGUARD_CONTEXT", lifeguard_context)
    def test_auth_header(self):
        header = "Basic YTpi dGVzdDpwYXNz"

        self.assertListEqual(
            basic_auth_login(header), [{"username": "test", "password": "pass"}]
        )

    def test_auth_invalid_user(self):
        header = "Basic YTpi"

        self.assertFalse(basic_auth_login(header), [])

    @patch("lifeguard.auth.FlaskResponse")
    @patch("lifeguard.auth.flask_request", spec={})
    @patch("lifeguard.auth.basic_auth_login")
    def test_basic_auth_login_required_implementation_call_function(
        self, mock_login, mock_request, _mock_response
    ):
        args = MagicMock(name="args")
        kwargs = MagicMock(name="kwargs")
        function = MagicMock(name="function")

        mock_request.headers = MagicMock(name="headers")
        mock_request.headers.get.return_value = "returned_get"

        mock_login.return_value = True

        function.return_value = False

        self.assertFalse(
            basic_auth_login_required_implementation(args, kwargs, function)
        )

        function.assert_called()
        mock_request.headers.get.assert_called_with("Authorization")
        mock_login.assert_called_with("returned_get")

    @patch("lifeguard.auth.FlaskResponse")
    @patch("lifeguard.auth.flask_request", spec={})
    @patch("lifeguard.auth.basic_auth_login")
    def test_basic_auth_login_required_implementation_return_401(
        self, mock_login, mock_request, mock_response
    ):
        args = MagicMock(name="args")
        kwargs = MagicMock(name="kwargs")
        function = MagicMock(name="function")

        mock_request.headers = MagicMock(name="headers")
        mock_request.headers.get.return_value = "returned_get"

        mock_login.return_value = False

        response = MagicMock(name="response")
        response.headers = {}
        mock_response.return_value = response

        self.assertEqual(
            basic_auth_login_required_implementation(args, kwargs, function),
            (response, 401),
        )

        self.assertDictEqual(response.headers, {"WWW-Authenticate": "Basic"})

        function.assert_not_called()
        mock_request.headers.get.assert_called_with("Authorization")
        mock_login.assert_called_with("returned_get")

    def test_authentication_methods(self):
        self.assertEqual(
            AUTHENTICATION_METHODS["basic_auth"],
            basic_auth_login_required_implementation,
        )



import os
import sys
import traceback
from functools import wraps

import jinja2
from flask import Blueprint
from flask import Response as FlaskResponse
from flask import request as flask_request

from lifeguard.auth import AUTHENTICATION_METHODS
from lifeguard.context import LIFEGUARD_CONTEXT
from lifeguard.logger import lifeguard_logger as logger
from lifeguard.settings import LIFEGUARD_DIRECTORY

custom_controllers = Blueprint("custom", __name__)


def render_template(template, searchpath, data=None):

    if not data:
        data = {}

    template_loader = jinja2.FileSystemLoader(searchpath=searchpath)
    template_env = jinja2.Environment(loader=template_loader)
    template = template_env.get_template(template)
    return template.render(**data)


class Request:
    @property
    def json(self):
        return flask_request.json

    @property
    def data(self):
        return flask_request.data

    @property
    def form(self):
        return flask_request.form

    @property
    def method(self):
        return flask_request.method

    @property
    def args(self):
        return flask_request.args

    @property
    def values(self):
        return flask_request.values


class Response:
    def __init__(self):
        self._content_type = "text/html"
        self._content = None
        self._template = None
        self._template_searchpath = None
        self._status = 200
        self._data = {}

    @property
    def content_type(self):
        return self._content_type

    @content_type.setter
    def content_type(self, value):
        self._content_type = value

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, value):
        self._template = value

    @property
    def template_searchpath(self):
        return self._template_searchpath

    @template_searchpath.setter
    def template_searchpath(self, value):
        self._template_searchpath = value

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        self._data = value


def login_required(function):
    """
    Decorator for login
    """

    @wraps(function)
    def wrapped(*args, **kwargs):

        if LIFEGUARD_CONTEXT.auth_method in AUTHENTICATION_METHODS:
            return AUTHENTICATION_METHODS[LIFEGUARD_CONTEXT.auth_method](
                args, kwargs, function
            )
        return function(*args, **kwargs)

    return wrapped


def register_custom_controller(path, function, options):
    endpoint = options.pop("endpoint", function.__name__)
    custom_controllers.add_url_rule(
        path, endpoint, configure_controller(login_required(function)), **options
    )


def load_custom_controllers():
    """
    Load custom controllers from application path
    """

    sys.path.append(LIFEGUARD_DIRECTORY)

    if not os.path.exists(os.path.join(LIFEGUARD_DIRECTORY, "controllers")):
        return

    for controller_file in os.listdir(os.path.join(LIFEGUARD_DIRECTORY, "controllers")):
        if controller_file.endswith("_controller.py"):
            controller_module_name = controller_file.replace(".py", "")
            logger.info("loading custom controller %s", controller_module_name)

            module = "controllers.%s" % (controller_module_name)
            if module not in sys.modules:
                __import__(module)


def treat_response(response):
    content = response.content
    if response.template:
        content = render_template(
            response.template, response.template_searchpath, data=response.data
        )

    return FlaskResponse(
        content, content_type=response.content_type, status=response.status
    )


def configure_controller(function):
    @wraps(function)
    def wrapped(*args, **kwargs):
        try:
            response = function(*args, **kwargs)

            if isinstance(response, Response):
                return treat_response(response)
            return response
        except Exception as error:
            logger.error(
                "error on render dashboard index: %s",
                str(error),
                extra={"traceback": traceback.format_exc()},
            )

    return wrapped


def controller(path, **options):
    """
    Decorator to configure a custom controller
    """

    def function_reference(decorated):
        @wraps(decorated)
        def wrapped(*args, **kwargs):
            return decorated(*args, **kwargs)

        register_custom_controller(path, decorated, options)
        return wrapped

    return function_reference


request = Request()



"""
Auth methods
"""
import base64

from flask import Response as FlaskResponse
from flask import request as flask_request

from lifeguard.context import LIFEGUARD_CONTEXT

BASIC_AUTH_METHOD = "basic_auth"


def basic_auth_login(authorization_header):
    """
    Basic auth method
    """
    encoded_uname_pass = authorization_header.split()[-1]
    username, password = base64.b64decode(encoded_uname_pass).decode("utf-8").split(":")
    return LIFEGUARD_CONTEXT.valid_user(username, password)


def basic_auth_login_required_implementation(args, kwargs, function):
    """
    Baic Auth Login Implementation
    """
    authorization_header = flask_request.headers.get("Authorization")
    if authorization_header and basic_auth_login(authorization_header):
        return function(*args, **kwargs)
    response = FlaskResponse()
    response.headers["WWW-Authenticate"] = "Basic"
    return response, 401


AUTHENTICATION_METHODS = {BASIC_AUTH_METHOD: basic_auth_login_required_implementation}



import unittest
from unittest.mock import MagicMock, patch

from lifeguard.notifications import (
    NOTIFICATION_METHODS,
    append_notification_implementation,
)


mock_logger = MagicMock(name="mock_logger")


class TestNotifications(unittest.TestCase):
    def setUp(self):
        NOTIFICATION_METHODS.clear()

    @patch("lifeguard.notifications.logger", mock_logger)
    def test_append_notification_implementation(self):
        implementation = MagicMock(name="implementation")
        implementation.__name__ = "implementation"
        implementation_instance = MagicMock(name="instance")
        implementation.return_value = implementation_instance

        append_notification_implementation(implementation)
        self.assertTrue(implementation_instance in NOTIFICATION_METHODS)
        mock_logger.info.assert_called_with(
            "appending implementation %s", implementation.__name__
        )

    @patch("lifeguard.notifications.logger", mock_logger)
    def test_append_notification_implementation_and_not_duplicate(self):
        class NotificationImplementation:
            pass

        append_notification_implementation(NotificationImplementation)
        append_notification_implementation(NotificationImplementation)

        self.assertEqual(1, len(NOTIFICATION_METHODS))



"""
Interface repositories
"""
from datetime import datetime

from lifeguard.logger import lifeguard_logger as logger

IMPLEMENTATIONS = {}


class BaseRepository(object):
    def __init_repository__(self, repository):
        self.__implementation__ = IMPLEMENTATIONS[repository]


class ValidationRepository(BaseRepository):
    def __init__(self):
        BaseRepository.__init_repository__(self, "validation")

    def save_validation_result(self, validation_result):
        validation_result.last_execution = datetime.now()
        self.__implementation__.save_validation_result(validation_result)

    def fetch_last_validation_result(self, validation_name):
        return self.__implementation__.fetch_last_validation_result(validation_name)

    def fetch_all_validation_results(self):
        return self.__implementation__.fetch_all_validation_results()


class NotificationRepository(BaseRepository):
    def __init__(self):
        BaseRepository.__init_repository__(self, "notification")

    def save_last_notification_for_a_validation(self, notification):
        self.__implementation__.save_last_notification_for_a_validation(notification)

    def fetch_last_notification_for_a_validation(self, validation_name):
        return self.__implementation__.fetch_last_notification_for_a_validation(
            validation_name
        )


def declare_implementation(repository, implementation):

    if repository in IMPLEMENTATIONS:
        logger.warning("overwriting implementation for respository %s", repository)
    logger.info(
        "loading implementation %s for repository %s",
        implementation.__name__,
        repository,
    )
    IMPLEMENTATIONS[repository] = implementation()



import os
import sys
import traceback
from json import JSONEncoder
from functools import wraps

from lifeguard.logger import lifeguard_logger as logger
from lifeguard.settings import (
    LIFEGUARD_DIRECTORY,
    LIFEGUARD_RUN_ONLY_VALIDATIONS,
    LIFEGUARD_SKIP_VALIDATIONS,
)

VALIDATIONS = {}


class ValidationResponse:
    """
    Represents the result of a validation
    """

    def __init__(
        self, validation_name, status, details, settings=None, last_execution=None
    ):
        self._validation_name = validation_name
        self._status = status
        self._details = details
        self._settings = settings
        self._last_execution = last_execution

    @property
    def validation_name(self):
        """
        Return validation name
        """
        return self._validation_name

    @property
    def status(self):
        """
        Return status
        """
        return self._status

    @property
    def details(self):
        """
        Return details
        """
        return self._details

    @property
    def settings(self):
        """
        Return settings
        """
        return self._settings

    @property
    def last_execution(self):
        """
        Return last execution
        """
        return self._last_execution

    @last_execution.setter
    def last_execution(self, value):
        """
        Setter for last execution
        """
        self._last_execution = value

    def get_attributes(self):
        """
        Return all attributes in a dict.
        """
        attributes = {
            "validation_name": self.validation_name,
            "status": self.status,
            "details": self.details,
            "settings": self.settings,
        }

        if self.last_execution:
            attributes["last_execution"] = self.last_execution.strftime(
                "%Y-%m-%dT%H:%M"
            )
        return attributes

    def __str__(self):
        return str(self.get_attributes())


class ValidationResponseEncoder(JSONEncoder):
    """
    Enconder for Validation Response
    """

    def default(self, validation_response):
        """
        Default implementation for validation response encoder
        """

        return validation_response.get_attributes()


def load_validations():
    """
    Load validations from application path
    """

    sys.path.append(LIFEGUARD_DIRECTORY)
    for validation_file in os.listdir(os.path.join(LIFEGUARD_DIRECTORY, "validations")):
        if validation_file.endswith("_validation.py"):
            validation_module_name = validation_file.replace(".py", "")
            logger.info("loading validation %s", validation_module_name)

            module = "validations.%s" % (validation_module_name)
            if module not in sys.modules:
                __import__(module)


def validation(description=None, actions=None, schedule=None, settings=None):
    """
    Decorator to configure a validation
    """

    def function_reference(decorated):
        @wraps(decorated)
        def wrapped(*args, **kwargs):
            try:

                if LIFEGUARD_RUN_ONLY_VALIDATIONS and (
                    decorated.__name__ not in LIFEGUARD_RUN_ONLY_VALIDATIONS
                ):
                    logger.info(
                        "validation %s not in LIFEGUARD_RUN_ONLY_VALIDATIONS",
                        decorated.__name__,
                    )
                    return None

                if LIFEGUARD_SKIP_VALIDATIONS and (
                    decorated.__name__ in LIFEGUARD_SKIP_VALIDATIONS
                ):
                    logger.info(
                        "validation %s in LIFEGUARD_SKIP_VALIDATIONS",
                        decorated.__name__,
                    )
                    return None

                result = decorated(*args, **kwargs)
                for action in actions or []:
                    logger.info(
                        "executing action %s with result %s...",
                        str(action.__name__),
                        str(result),
                    )
                    action(result, settings)

                return result
            except Exception as exception:
                logger.warning(
                    "validation error %s: %s",
                    str(decorated.__name__),
                    str(exception),
                    extra={"traceback": traceback.format_exc()},
                )

        VALIDATIONS[decorated.__name__] = {
            "ref": wrapped,
            "description": description,
            "actions": actions,
            "schedule": schedule,
            "settings": settings,
        }

        return wrapped

    return function_reference
